@@ -67,6 +67,27 @@ def test_predefined(solver):
     assert np.allclose(yout[-1, :], [-1.89021896, -0.71633577])
 
 
+def test_to_arrays():
+    odesys1 = ODESys(vdp_f, vdp_j, vdp_dfdt)
+    assert [e.tolist() for e in odesys1.to_arrays(3, [4, 5], [6])] == [[0, 3], [4, 5], [6]]
+    assert [e.tolist() for e in odesys1.to_arrays([2, 3], [4, 5], [6])] == [[2, 3], [4, 5], [6]]
+    assert [e.tolist() for e in odesys1.to_arrays([1, 2, 3], [4, 5], [6])] == [[1, 2, 3], [4, 5], [6]]
+    assert [e.tolist() for e in odesys1.to_arrays([[1, 2, 3], [0, 1, 2]], [4, 5], [6])] == [
+        [[1, 2, 3], [0, 1, 2]], [[4, 5]]*2, [[6]]*2]
+    assert [e.tolist() for e in odesys1.to_arrays([1, 2, 3], [[4, 4], [5, 5], [6, 6]], [6])] == [
+        [[1, 2, 3]]*3, [[4, 4], [5, 5], [6, 6]], [[6]]*3]
+    odesys2 = ODESys(vdp_f, vdp_j, vdp_dfdt, names='A B'.split(), dep_by_name=True)
+    assert [e.tolist() for e in odesys2.to_arrays(3, {'A': 4, 'B': 5}, [6])] == [[0, 3], [4, 5], [6]]
+    assert [e.tolist() for e in odesys2.to_arrays([2, 3], {'A': 4, 'B': 5}, [6])] == [[2, 3], [4, 5], [6]]
+    assert [e.tolist() for e in odesys2.to_arrays([1, 2, 3], {'A': 4, 'B': 5}, [6])] == [[1, 2, 3], [4, 5], [6]]
+    assert [e.tolist() for e in odesys2.to_arrays([[1, 2, 3], [0, 1, 2]], {'A': 4, 'B': 5}, [6])] == [
+        [[1, 2, 3], [0, 1, 2]], [[4, 5]]*2, [[6]]*2]
+    assert [e.tolist() for e in odesys2.to_arrays([1, 2, 3], {'A': [4, 5, 6], 'B': [4, 5, 6]}, [6])] == [
+        [[1, 2, 3]]*3, [[4, 4], [5, 5], [6, 6]], [[6]]*3]
+    assert [e.tolist() for e in odesys2.to_arrays([1, 2, 3], {'A': [4, 5, 6], 'B': 8}, [6])] == [
+        [[1, 2, 3]]*3, [[4, 8], [5, 8], [6, 8]], [[6]]*3]
+
+
 @requires('scipy')
 def test_pre_post_processors():
     """
@@ -152,12 +173,13 @@ def _test_integrate_multiple_predefined(odes, **kwargs):
     _xout = np.array([[0, 1, 2], [0, 1, 2], [0, 1, 2]])
     _y0 = np.array([[1, 2], [2, 3], [3, 4]])
     _params = np.array([[5], [6], [7]])
-    xout, yout, info = odes.integrate(_xout, _y0, params=_params, **kwargs)
+    results = odes.integrate(_xout, _y0, params=_params, **kwargs)
     for idx in range(3):
-        ref = _y0[idx, 0]*np.exp(-_params[idx, 0]*xout[idx, :])
-        assert np.allclose(yout[idx, :, 0], ref)
-        assert np.allclose(yout[idx, :, 1], _y0[idx, 0] - ref + _y0[idx, 1])
-        assert info[idx]['nfev'] > 0
+        xout, yout, info = results[idx]
+        ref = _y0[idx, 0]*np.exp(-_params[idx, 0]*xout)
+        assert np.allclose(yout[:, 0], ref)
+        assert np.allclose(yout[:, 1], _y0[idx, 0] - ref + _y0[idx, 1])
+        assert info['nfev'] > 0
 
 
 @requires('scipy')
@@ -172,7 +194,8 @@ def test_integrate_multiple_predefined__cvode():
 
 @requires('pyodeint')
 def test_integrate_multiple_predefined__odeint():
-    _test_integrate_multiple_predefined(ODESys(decay), integrator='odeint', method='bulirsch_stoer', atol=1e-9)
+    _test_integrate_multiple_predefined(ODESys(decay), integrator='odeint', method='bulirsch_stoer',
+                                        atol=1e-10, rtol=1e-10)
 
 
 @requires('pygslodeiv2')
@@ -242,12 +265,13 @@ def _test_integrate_multiple_adaptive(odes, **kwargs):
     def _ref(A, k, t):
         return [A*np.sin(k*(t - t[0])), A*np.cos(k*(t - t[0]))*k]
 
-    xout, yout, info = odes.integrate(_xout, _y0, _params, **kwargs)
+    results = odes.integrate(_xout, _y0, _params, **kwargs)
     for idx in range(3):
-        ref = _ref(Ak[idx][0], Ak[idx][1], xout[idx])
-        assert np.allclose(yout[idx][:, 0], ref[0], atol=1e-5, rtol=1e-5)
-        assert np.allclose(yout[idx][:, 1], ref[1], atol=1e-5, rtol=1e-5)
-        assert info[idx]['nfev'] > 0
+        xout, yout, info = results[idx]
+        ref = _ref(Ak[idx][0], Ak[idx][1], xout)
+        assert np.allclose(yout[:, 0], ref[0], atol=1e-5, rtol=1e-5)
+        assert np.allclose(yout[:, 1], ref[1], atol=1e-5, rtol=1e-5)
+        assert info['nfev'] > 0
 
 
 @requires('scipy')
@@ -291,6 +315,7 @@ def decay_factory(ny):
     return f, j, dfdt
 
 
+@pytest.mark.slow
 @requires('pyodeint', 'scipy')
 def test_par_by_name__multi():
     from scipy.special import binom
@@ -305,11 +330,11 @@ def test_par_by_name__multi():
         ref = np.array([[binom(p + idx, p)*(a/(a+1))**idx/(a+1)**(p+1) for idx in range(ny)]
                         for p in range(p_max + 1)])
         odesys = ODESys(*decay_factory(ny), param_names=params.keys(), par_by_name=True)
-        result = odesys.integrate(np.linspace(0, 1), [1] + [0]*(ny-1), params,
-                                  integrator='odeint', method='rosenbrock4')
-        assert all(nfo['success'] for nfo in result.info)
-        assert result.xout.shape[-1] == 50
-        assert np.allclose(result.yout[:, -1, :], ref)
+        results = odesys.integrate(np.linspace(0, 1), [1] + [0]*(ny-1), params,
+                                   integrator='odeint', method='rosenbrock4')
+        assert all(r.info['success'] for r in results)
+        assert all(r.xout.shape[-1] == 50 for r in results)
+        assert all(np.allclose(r.yout[-1, :], ref[i, ...]) for i, r in enumerate(results))
 
 
 @requires('pygslodeiv2')
@@ -318,30 +343,30 @@ def test_par_by_name__multi__single_varied():
     odesys1 = ODESys(*decay_factory(ny), param_names='a b c'.split(), par_by_name=True)
     params1 = {'a': 2, 'b': (3, 4, 5, 6, 7), 'c': 0}
     init1 = [42, 0, 0]
-    result1 = odesys1.integrate(2.1, init1, params1, integrator='gsl')
+    results1 = odesys1.integrate(2.1, init1, params1, integrator='gsl')
     for idx1 in range(len(params1['b'])):
-        ref_a1 = init1[0]*np.exp(-params1['a']*result1.xout[idx1])
+        ref_a1 = init1[0]*np.exp(-params1['a']*results1[idx1].xout)
         ref_b1 = init1[0]*params1['a']*(
-            np.exp(-params1['a']*result1.xout[idx1]) -
-            np.exp(-params1['b'][idx1]*result1.xout[idx1]))/(params1['b'][idx1] - params1['a'])
+            np.exp(-params1['a']*results1[idx1].xout) -
+            np.exp(-params1['b'][idx1]*results1[idx1].xout))/(params1['b'][idx1] - params1['a'])
         ref_c1 = init1[0] - ref_a1 - ref_b1
-        assert np.allclose(result1.yout[idx1][:, 0], ref_a1)
-        assert np.allclose(result1.yout[idx1][:, 1], ref_b1)
-        assert np.allclose(result1.yout[idx1][:, 2], ref_c1)
+        assert np.allclose(results1[idx1].yout[:, 0], ref_a1)
+        assert np.allclose(results1[idx1].yout[:, 1], ref_b1)
+        assert np.allclose(results1[idx1].yout[:, 2], ref_c1)
 
     odesys2 = ODESys(*decay_factory(ny), names='a b c'.split(), dep_by_name=True)
     init2 = {'a': (7, 13, 19, 23, 42, 101), 'b': 0, 'c': 0}
     params2 = [11.7, 12.3, 0]
-    result2 = odesys2.integrate(3.4, init2, params2, integrator='gsl')
+    results2 = odesys2.integrate(3.4, init2, params2, integrator='gsl')
     for idx2 in range(len(init2['a'])):
-        ref_a2 = init2['a'][idx2]*np.exp(-params2[0]*result2.xout[idx2])
+        ref_a2 = init2['a'][idx2]*np.exp(-params2[0]*results2[idx2].xout)
         ref_b2 = init2['a'][idx2]*params2[0]*(
-            np.exp(-params2[0]*result2.xout[idx2]) -
-            np.exp(-params2[1]*result2.xout[idx2]))/(params2[1] - params2[0])
+            np.exp(-params2[0]*results2[idx2].xout) -
+            np.exp(-params2[1]*results2[idx2].xout))/(params2[1] - params2[0])
         ref_c2 = init2['a'][idx2] - ref_a2 - ref_b2
-        assert np.allclose(result2.yout[idx2][:, 0], ref_a2)
-        assert np.allclose(result2.yout[idx2][:, 1], ref_b2)
-        assert np.allclose(result2.yout[idx2][:, 2], ref_c2)
+        assert np.allclose(results2[idx2].yout[:, 0], ref_a2)
+        assert np.allclose(results2[idx2].yout[:, 1], ref_b2)
+        assert np.allclose(results2[idx2].yout[:, 2], ref_c2)
 
 
 @requires('scipy')
@@ -398,12 +423,11 @@ def test_roots():
 def test_quantities_param():
     import quantities as pq
 
-    def pp(x, y, p):
-        return x, y, [p[0].rescale(1/pq.s).magnitude]
-
-    odesys = ODESys(sine, sine_jac, param_names=['k'], par_by_name=True, pre_processors=[pp])
+    odesys = ODESys(sine, sine_jac, param_names=['k'], par_by_name=True, to_arrays_callbacks=(
+        None, None, lambda p: [p[0].rescale(1/pq.s).magnitude]))
     A, k = 2, 3
-    xout, yout, info = odesys.integrate(np.linspace(0, 1), [0, A*k], {'k': k/pq.second})
+    xout, yout, info = odesys.integrate(np.linspace(0, 1), [0, A*k],
+                                        {'k': k/pq.second})
     assert info['success']
     assert xout.size > 7
     ref = [
@@ -412,3 +436,62 @@ def test_quantities_param():
     ]
     assert np.allclose(yout[:, 0], ref[0], atol=1e-5, rtol=1e-5)
     assert np.allclose(yout[:, 1], ref[1], atol=1e-5, rtol=1e-5)
+
+
+@requires('quantities')
+def test_quantities_param_multi():
+    import quantities as pq
+
+    def pp(x, y, p):
+        return x, y, np.array([[item.rescale(1/pq.s).magnitude
+                                for item in p[:, idx]] for idx in range(1)]).T
+
+    odesys = ODESys(sine, sine_jac, param_names=['k'],
+                    par_by_name=True, pre_processors=[pp])
+    A = 2.
+    kvals = (7452., 13853., 22123.)
+    results = odesys.integrate(
+        np.linspace(0, 1), [[0, A*kval/3600] for kval in kvals],
+        {'k': [val/pq.hour for val in kvals]}
+    )
+    assert len(results) == 3
+    assert all([r.info['success'] for r in results])
+    for res, kval in zip(results, kvals):
+        assert res.xout.size > 7
+        ref = [
+            A*np.sin(1/3600*kval*(res.xout - res.xout[0])),
+            A*np.cos(1/3600*kval*(res.xout - res.xout[0]))*kval/3600
+        ]
+        assert np.allclose(res.yout[:, 0], ref[0], atol=1e-5, rtol=1e-5)
+        assert np.allclose(res.yout[:, 1], ref[1], atol=1e-5, rtol=1e-5)
+
+
+@requires('quantities')
+def test_quantities_param_multi2():
+    import quantities as pq
+
+    units = [1/pq.s]
+    odesys = ODESys(
+        sine, sine_jac, param_names=['k'], par_by_name=True, to_arrays_callbacks=(
+            None, None, lambda p: np.array([
+                [elem.rescale(u).magnitude for elem in parvals]
+                for parvals, u in zip(p.T, units)
+            ]).T
+        )
+    )
+    A = 2.
+    kvals = (7452., 13853., 22123.)
+    results = odesys.integrate(
+        np.linspace(0, 1), [[0, A*kval/3600] for kval in kvals],
+        {'k': kvals/pq.hour}
+    )
+    assert len(results) == 3
+    assert all([r.info['success'] for r in results])
+    for res, kval in zip(results, kvals):
+        assert res.xout.size > 7
+        ref = [
+            A*np.sin(1/3600*kval*(res.xout - res.xout[0])),
+            A*np.cos(1/3600*kval*(res.xout - res.xout[0]))*kval/3600
+        ]
+        assert np.allclose(res.yout[:, 0], ref[0], atol=1e-5, rtol=1e-5)
+        assert np.allclose(res.yout[:, 1], ref[1], atol=1e-5, rtol=1e-5)
